from io import BytesIO
from pathlib import Path
import subprocess
import gzip
import os
from pydbbackups.errors import CMDExecutionError
from .base import Backup

DUMP_DEFAULT_FORMAT = "sql"
DUMP_SQL_FORMAT = "sql"
DUMP_GZIP_FORMAT = "gz"


class MySQL(Backup):
    cmds_to_check = [('mysqldump', '--version'), ('mysql', '--version')]

    def dump(self, **kwargs) -> BytesIO:
        env = os.environ.copy()
        env['MYSQL_PWD'] = self.password

        args = [
            '--quick',
            f'--host={self.host}',
            f'--port={self.port}',
            f"--user={self.username}",
            '--databases',
            self.database,
            '--force'
        ]

        with subprocess.Popen(
            [
                'mysqldump',
                *args
            ], env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        ) as process:
            status_code = process.wait()

            if status_code > 0:
                error = process.stderr.read().decode('utf-8')
                raise CMDExecutionError(error)

            output = BytesIO(process.stdout.read())

        if kwargs.get('compress') is True:
            output = BytesIO(gzip.compress(output.read()))

        return output

    def restore(self, file_path: Path) -> BytesIO:
        env = os.environ.copy()
        env['MYSQL_PWD'] = self.password

        args = [
            '-h',
            self.host,
            '-P',
            f"{self.port}",
            '-u',
            self.username,
            '-e'
        ]

        if file_path.name.endswith(DUMP_GZIP_FORMAT):
            sql_content = gzip.decompress(file_path.resolve().read_bytes())
            backup = sql_content.decode('utf-8')
        else:
            backup = f'source {file_path.resolve()}'

        args.append(backup)

        with subprocess.Popen(
            [
                'mysql',
                *args
            ], env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        ) as process:
            status_code = process.wait()

            if status_code > 0:
                error = process.stderr.read().decode('utf-8')
                raise CMDExecutionError(error)



from dataclasses import dataclass
from typing import Optional, Sequence, Tuple
from io import BytesIO
from pathlib import Path
from shutil import which
from pydbbackups.errors import MethodNotImplemented, CommandNotFound


@dataclass
class Backup:
    host: str
    username: str
    database: str

    uri: Optional[str] = None
    password: Optional[str] = None
    port: Optional[int] = None

    cmds_to_check: Optional[Sequence[Tuple[str, Optional[str]] | str]] = None

    @classmethod
    def exist_cmds(cls):
        if not cls.cmds_to_check:
            print(
                f"Warning : The {cls.__class__.__name__} class doesn't have commands to check")
            return
        # pylint: disable=E1133
        for cmd in cls.cmds_to_check:
            # args = []
            if isinstance(cmd, tuple):
                command = cmd[0]
                # if len(cmd) > 1:
                #     args = cmd[1]
            else:
                command = cmd

            if not which(command):
                raise CommandNotFound(command)

    def dump(self, **kwargs) -> BytesIO:
        """ Generates a backup """
        raise MethodNotImplemented('Dump')

    def restore(self, file_path: Path) -> BytesIO:
        """ Restore a backup """
        raise MethodNotImplemented('Restore')

    def __post_init__(self):
        self.exist_cmds()
